SystemOrganization addCategory: #'Rest-Client-HTTP'!
SystemOrganization addCategory: #'Rest-Client-Proxies'!

!Dictionary methodsFor: '*rest-client' stamp: 'mt 2/16/2009 10:28'!
keysAndValuesCollect: aBlock 
	| newDict |
	newDict := Dictionary new.
	self keysAndValuesDo: [:key :value | newDict add: (aBlock value: key value: value)].
	^ newDict! !

SequenceableCollection subclass: #RestProxyCollection
	instanceVariableNames: 'baseUri factory proxyObjects restUri'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-Proxies'!

!RestProxyCollection class methodsFor: 'instance creation' stamp: 'mt 3/5/2009 17:15'!
fromServer: aBaseUriString path: aPathString 

	^ self new
		restUri: aPathString;
		baseUri: aBaseUriString! !

!RestProxyCollection methodsFor: 'as yet unclassified' stamp: 'cs 3/11/2009 17:12'!
add: aRestObject 

	| newObject |
	newObject := (aRestObject id isNil or: [aRestObject id < 1]) 
		ifTrue: [self factory addObject: self restUri with: aRestObject]
		ifFalse: [self factory addObject: self restUri with: aRestObject restUri].
	self proxyObjects add: newObject.
	self changed.
	^ newObject! !

!RestProxyCollection methodsFor: 'as yet unclassified' stamp: 'cs 3/11/2009 17:26'!
addNew: aPropertyDict  

	| newObject |
	newObject := self factory addObject: self restUri with: aPropertyDict.
	self proxyObjects add: newObject.
	self changed.
	^ newObject
! !

!RestProxyCollection methodsFor: 'as yet unclassified' stamp: 'mt 3/5/2009 16:01'!
at: index

	^ self proxyObjects at: index! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'mt 3/6/2009 13:20'!
baseUri

	^ baseUri! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'mt 3/6/2009 13:20'!
baseUri: anObject

	baseUri := anObject! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'mt 3/6/2009 13:20'!
factory

	^ factory! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'mt 3/6/2009 13:20'!
factory: anObject

	factory := anObject! !

!RestProxyCollection methodsFor: 'as yet unclassified' stamp: 'mt 3/5/2009 16:46'!
fill: someRestUris
	"Take list of rest uris and create proxy objects to fill this collection."
	
	self proxyObjects: (someRestUris collect: [:path | self factory getObject: path]) asOrderedCollection! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'mt 3/6/2009 13:21'!
proxyObjects

	^ proxyObjects! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'mt 3/6/2009 13:21'!
proxyObjects: anObject

	proxyObjects := anObject! !

!RestProxyCollection methodsFor: 'as yet unclassified' stamp: 'cs 3/13/2009 10:39'!
remove: aRestObject ifAbsent: aBlock

	self proxyObjects remove: aRestObject ifAbsent: aBlock.	
	RestSocket
		deleteDocumentFromUrl: ('{1}{2}/{3}' format: {self baseUri. self restUri. aRestObject id}) asUrl
		request: ''.
	self changed.
	^ aRestObject! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'mt 3/6/2009 13:21'!
restUri

	^ restUri! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'mt 3/6/2009 13:21'!
restUri: anObject 

	restUri := anObject! !

!RestProxyCollection methodsFor: 'as yet unclassified' stamp: 'mt 3/5/2009 16:02'!
size

	^ self proxyObjects size! !

!RestProxyCollection methodsFor: 'as yet unclassified' stamp: 'mt 3/6/2009 13:03'!
species

	^ Array! !

Object subclass: #RestProxyObject
	instanceVariableNames: 'properties socket baseUri factory restUri'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-Proxies'!

!RestProxyObject commentStamp: 'mt 3/2/2009 16:14' prior: 0!
I am a proxy object. I store a reference to the real object in form of a url (e.g. http://bp2008h1srv:8082/projects/5/tasks/3). I behave similar to the real object because I understand its accessors.!

!RestProxyObject class methodsFor: 'instance creation' stamp: 'mt 3/12/2009 11:42'!
fromServer: aBaseUriString path: aPathString 

	^ RestProxyObject allInstances
		detect: [:proxyObject | "proxyObject baseUri = aBaseUriString" false]
		ifNone: [self new
			restUri: aPathString;
			baseUri: aBaseUriString]! !

!RestProxyObject class methodsFor: 'as yet unclassified' stamp: 'topa 3/9/2009 16:57'!
toProperty: aSymbol

 	| selector |
	selector := aSymbol withFirstCharacterDownshifted.
	^ (selector endsWith: 'y') 
		ifTrue: [selector allButLast, 'ies']
		ifFalse: [selector, 's']
	! !

!RestProxyObject methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:46'!
baseUri

	^ baseUri! !

!RestProxyObject methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:46'!
baseUri: anObject

	baseUri := anObject.! !

!RestProxyObject methodsFor: 'updating' stamp: 'mt 3/5/2009 17:00'!
commit
	^ RestSocket 
		putDocumentFromUrl: (self baseUri , self restUri) asUrl
		request: ''
		content: (Json render: self properties)! !

!RestProxyObject methodsFor: 'updating' stamp: 'mt 3/5/2009 17:00'!
commit: key 
	^ RestSocket 
		putDocumentFromUrl: (self baseUri , self restUri) asUrl
		request: ''
		content: (Json render: (Dictionary with: key -> (self properties at: key)))! !

!RestProxyObject methodsFor: 'deleting' stamp: 'mt 3/5/2009 17:00'!
delete
	RestSocket deleteDocumentFromUrl: (self baseUri , self restUri) asUrl
		request: ''! !

!RestProxyObject methodsFor: 'error handling' stamp: 'topa 3/9/2009 16:08'!
doesNotUnderstand: aMessage

	" handle getters"
	(self properties includesKey: aMessage selector)
		ifTrue: [^ self properties at: aMessage selector].

	"handle add-remove"
	((aMessage selector startsWith: 'add') and: [aMessage selector endsWith: ':'])
		ifTrue: [^ self handleAddMessage: aMessage].
		
	" handle setters"
	((aMessage selector endsWith: ':') and:
		[self properties includesKey: aMessage selector allButLast])
			ifTrue: [^ self set: aMessage selector allButLast value: aMessage arguments first].

	"handle unknown"
	super doesNotUnderstand: aMessage.! !

!RestProxyObject methodsFor: 'accessing' stamp: 'topa 3/3/2009 18:19'!
factory

	^ factory! !

!RestProxyObject methodsFor: 'accessing' stamp: 'topa 3/3/2009 18:19'!
factory: anObject

	factory := anObject! !

!RestProxyObject methodsFor: 'accessing' stamp: 'topa 3/9/2009 17:43'!
handleAddMessage: aMessage
	
	| target property targetSelector |
	target :=  aMessage selector allButFirst: 3.
	
	targetSelector := (target startsWith: 'New') 
		ifTrue: [target :=  target allButFirst: 3. #addNew:]
		ifFalse:  [#add:].
	
	property := self properties 
		at: (self class toProperty: target allButLast) 
		ifAbsent: [^ super doesNotUnderstand: aMessage].
		
	^ property perform: targetSelector withArguments: aMessage arguments
! !

!RestProxyObject methodsFor: 'accessing' stamp: 'topa 3/3/2009 17:40'!
properties

	^ properties ifNil: [self reload. properties]! !

!RestProxyObject methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:47'!
properties: anObject

	properties := anObject.! !

!RestProxyObject methodsFor: 'updating' stamp: 'SR 3/9/2009 14:59'!
reload
	| jsonResult response |
	response := RestSocket get: self baseUri , self restUri.
	(response responseCode = 200) ifFalse: [
		self error: ('Error retrieving contents from server. HTTP response code was {1}.' format: {response responseCode})
	].

	jsonResult := Json readFrom: response content readStream.

	self 
		properties: (jsonResult keysAndValuesCollect: 
					[:key :value | 
					value isArray 
						ifFalse: [key -> value]
						ifTrue: 
							[key -> ((RestProxyCollection fromServer: self baseUri path: ('{1}/{2}' format: {self restUri. key}))
										factory: self factory;
										fill: value)]])! !

!RestProxyObject methodsFor: 'accessing' stamp: 'mt 3/5/2009 17:01'!
restUri
	^ restUri! !

!RestProxyObject methodsFor: 'accessing' stamp: 'mt 3/5/2009 17:01'!
restUri: anObject 
	restUri := anObject! !

!RestProxyObject methodsFor: 'accessing' stamp: 'cs 3/11/2009 17:09'!
set: property value: value

	self properties at: property put: (value isText 
		ifTrue: [value asString] ifFalse: [value]).
	self commit: property.
	self changed: property.! !

Object subclass: #RestProxyObjectFactory
	instanceVariableNames: 'url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-Proxies'!

!RestProxyObjectFactory commentStamp: 'mt 3/2/2009 16:39' prior: 0!
I provide convenient access to a rest server by creating rest proxy objects.!

!RestProxyObjectFactory class methodsFor: 'instance creation' stamp: 'topa 3/5/2009 15:30'!
fromString: aServerString
	"e.g. http://bp2008h1srv:8082/"

	^ self new url: aServerString asUrl! !

!RestProxyObjectFactory methodsFor: 'instance creation' stamp: 'topa 3/5/2009 15:31'!
addObject: path with: anObject 

	| response |
	response := RestSocket 
				postDocumentFromUrl: (self url asString, path) asUrl
				request: ''
				content: (Json render: anObject).
	^ self getObject: response content! !

!RestProxyObjectFactory methodsFor: 'instance creation' stamp: 'mt 3/5/2009 17:29'!
getCollection: path 
	"Get list of cn's and request their Objects into a Collection"
	
	| jsonCollection |
	jsonCollection :=  Json readFrom: (RestSocket get: self url asString, path) content readStream.
	^ (RestProxyCollection fromServer: self url asString path: path)
		factory: self;
		fill: jsonCollection! !

!RestProxyObjectFactory methodsFor: 'instance creation' stamp: 'topa 3/5/2009 15:31'!
getObject: path 

	^ (RestProxyObject fromServer: self url asString path: path)
		factory: self! !

!RestProxyObjectFactory methodsFor: 'accessing' stamp: 'topa 3/5/2009 15:30'!
url

	^ url! !

!RestProxyObjectFactory methodsFor: 'accessing' stamp: 'topa 3/5/2009 15:30'!
url: anObject

	url := anObject.! !

Object subclass: #RestResponse
	instanceVariableNames: 'content headers responseCode url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-HTTP'!

!RestResponse class methodsFor: 'instance creation' stamp: 'mfp 2/20/2009 16:05'!
responseCode: aNumberCode headers: aDictionary content: aString url: aUrl

	^ self new
		responseCode: aNumberCode asNumber;
		headers: aDictionary;
		content: aString;
		url: aUrl! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
content

	^ content! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
content: anObject

	content := anObject.! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:49'!
getHeader: name 

	^ self getHeader: name default: nil! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
getHeader: name  default: defaultValue

	^ self headers at: name  ifAbsent: [defaultValue]! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
headers

	^ headers! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
headers: anObject

	headers := anObject.! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
responseCode

	^ responseCode! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
responseCode: anObject

	responseCode := anObject.! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
url

	^ url! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:31'!
url: anObject

	url := anObject.! !

Object subclass: #RestSocket
	instanceVariableNames: 'conn headers responseCode request content url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-HTTP'!

!RestSocket class methodsFor: 'private' stamp: 'topa 3/3/2009 21:40'!
defaultPortForSchema: aSchema
	"default port to connect on"

	aSchema = 'http' ifTrue: [^ 80].
	aSchema = 'https' ifTrue: [^ 443].! !

!RestSocket class methodsFor: 'requesting' stamp: 'topa 3/3/2009 21:40'!
deleteDocumentFromUrl: url request: requestString 

	^ self 
		doHTTPRequestFromUrl: url
		verb: 'DELETE'
		request: requestString
		content: nil! !

!RestSocket class methodsFor: 'requesting' stamp: 'topa 2/25/2009 11:11'!
doHTTPRequestFromUrl: url verb: aVerb request: requestString content: aContentString 
	| serverName sock |
	self flag: #NeedsRefactoring.
	Socket initializeNetwork.

	serverName := url authority.
	3 timesRepeat: [
		sock := self new 
			url: url;
			request: requestString;
			content: aContentString.
		(sock connectAndWaitUntil: (Time millisecondClockValue + 10000 truncated))
			ifTrue: [^ sock doVerb: aVerb]].
	Socket deadServer: serverName.
	sock destroy.
	^ 'Server ' , serverName , ' is not responding'! !

!RestSocket class methodsFor: 'requesting' stamp: 'topa 3/3/2009 21:41'!
expandUrl: newUrl ip: byteArrayIP port: portNum

	self flag: #SplitUp; flag: #NeedsRefactoring.
	(newUrl beginsWith: '../') 
		ifTrue: [
			^ String streamContents: [:strm | 
				byteArrayIP do: [:bb | bb printOn: strm.  strm nextPut: $.].
				strm skip: -1; nextPut: $:.
				portNum printOn: strm.
				strm nextPutAll: (newUrl allButFirst: 2)]].
	(newUrl beginsWith: '/') 
		ifTrue: [
			^ String streamContents: [:strm | 
				byteArrayIP do: [:bb | bb printOn: strm.  strm nextPut: $.].
				strm skip: -1; nextPut: $:.
				portNum printOn: strm.
				strm nextPutAll: newUrl]].
	^ newUrl
! !

!RestSocket class methodsFor: 'requesting' stamp: 'SR 2/26/2009 14:59'!
get: aUriString
	"Send an HTTP GET request."

	^ RestSocket 
		getDocumentFromUrl: aUriString asUrl "for Socket compatibility"
		request: ''! !

!RestSocket class methodsFor: 'requesting' stamp: 'topa 3/3/2009 21:41'!
getDocumentFromUrl: url request: requestString 

	^ self 
		doHTTPRequestFromUrl: url
		verb: 'GET'
		request: requestString
		content: nil! !

!RestSocket class methodsFor: 'requesting' stamp: 'topa 3/3/2009 21:41'!
postDocumentFromUrl: url request: requestString content: contentString 

	^ self 
		doHTTPRequestFromUrl: url
		verb: 'POST'
		request: requestString
		content: contentString! !

!RestSocket class methodsFor: 'requesting' stamp: 'topa 3/3/2009 21:41'!
putDocumentFromUrl: url request: requestString content: contentString 

	^ self 
		doHTTPRequestFromUrl: url
		verb: 'PUT'
		request: requestString
		content: contentString! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:31'!
conn

	^ conn! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:31'!
conn: anObject

	conn := anObject.! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:32'!
connect
	| serverAddr |
	serverAddr := (NetNameResolver addressForName: self serverName timeout: 20)
		ifNil: [self error: 'Could not resolve the server named: ', self serverName].
		
	self connectTo: serverAddr port: self port.! !

!RestSocket methodsFor: 'api' stamp: 'topa 2/19/2009 19:41'!
connectAndWaitUntil: aTimeout

	^ self connect; waitForConnectionUntil: aTimeout! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:32'!
connectTo: addr port: aPort

	self conn: (self usesSSL
			ifTrue: [SSLSocket newConnectionAddress: addr port: aPort version: SSLProtocolVersion tls10]
			ifFalse: [Socket new connectTo: addr port: aPort]).
! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:32'!
content

	^ content! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:32'!
content: anObject

	content := anObject.! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:32'!
destroy

	self conn close.! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/5/2009 15:45'!
doVerb: aHTTPVerb
	| cmd stream |
	self flag: #NeedsRefactoring.
	cmd := 	aHTTPVerb,' ', self url fullPath, ' HTTP/1.0', String crlf, 
				'Host: ', self serverName , ':', self port printString, String crlf, 
				self userNamePasswordIfExists,
				(self request ifEmpty: [''] ifNotEmpty: [self request, String crlf]).
					
	self content ifNotNil: [cmd := cmd, 
		'Content-type: application/json', String crlf,
		'Content-length: ', self content size printString , String crlf].
	self sendCommand: cmd.

	"headers are sent."	
	self content ifNotNil: [self sendCommand: self content].
	stream := self getData readStream.
	self headers: (stream upToAll: String crlfcrlf). "parses header, too"
	^ RestResponse 
			responseCode: self responseCode  
			headers: self headers
			content: (self getRestOfData: stream upToEnd)
			url: self url! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:35'!
getData

	| data |
	data := self usesSSL
		ifTrue: [self conn receive]
		ifFalse: [self conn receiveData].
	[self conn isConnected and: [(data asString findString: String crlfcrlf) == 0]]
		whileTrue: [
			(Delay forMilliseconds: 100) wait.
				data := data, (self usesSSL
					ifTrue: [self conn receive]
					ifFalse: [self conn receiveData])].
	^ data asString! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:36'!
getHeader: name 

	^ self getHeader: name  default: nil! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:36'!
getHeader: name  default: defaultValue

	^ self headers at: name  ifAbsent: [defaultValue]! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:37'!
getRestOfData: begin

	| data length |
	self flag: #NeedsRefactoring.
	length := self getHeader: 'content-length'.
	length ifNotNil: [ length := length asNumber ].
	data := begin, (self usesSSL
		ifTrue: [self conn receive]
		ifFalse: [self conn receiveAvailableData]).
	[self conn isConnected and: [length isNil or: [data size < length]]]
		whileTrue: [
			(Delay forMilliseconds: 100) wait.
				data := data, (self usesSSL
					ifTrue: [self conn receive]
					ifFalse: [self conn receiveAvailableData]).].
	^ data asString! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:37'!
headers

	^ headers! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:37'!
headers: headerText
	"set the headers.  Then getHeader: can be used"

	| lines foldedLines i statusLine |
	self flag: #SplitUp.
	"divide into basic lines"
	lines := headerText findTokens: (String with: Character cr  with: Character linefeed).
	statusLine := lines first.
	lines := lines copyFrom: 2 to: lines size.

	"parse the status (pretty trivial right now)"
	responseCode := (statusLine findTokens: ' ') second.

	"fold lines that start with spaces into the previous line"
	foldedLines := OrderedCollection new.
	lines do: [ :line |
		line first isSeparator ifTrue: [
			foldedLines at: foldedLines size  put: (foldedLines last, line) ]
		ifFalse: [ foldedLines add: line ] ].

	"make a dictionary mapping headers to header contents"
	headers := Dictionary new.
	foldedLines do: [ :line |
		i := line indexOf: $:.
		i > 0 ifTrue: [
			self headers 
			at: (line copyFrom: 1 to: i-1) asLowercase 
			put: (line copyFrom: i+1 to: line size) withBlanksTrimmed ] ].
! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:38'!
isConnecting

	^ self usesSSL
		ifTrue: [self conn isConnecting]
		ifFalse: [self conn isWaitingForConnection]! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:38'!
port

	^ self url port 
		ifNil:  [self class defaultPortForSchema: self url schemeName]
		ifNotNil: [self url port]
! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:38'!
request

	^ request! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:38'!
request: anObject

	request := anObject.! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:38'!
responseCode

	^ responseCode! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:38'!
sendCommand: cmd
	
	self usesSSL
		ifTrue: [self conn send: cmd, String crlf]
		ifFalse: [self conn sendData: cmd, String crlf]! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:39'!
serverName

	^ self url authority! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:39'!
url

	^ url! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:39'!
url: anObject

	url := anObject.! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/5/2009 15:39'!
userNamePasswordIfExists

	^ (self url username notNil) & (self url password notNil)
		ifFalse: ['']
		ifTrue: ['Authorization: Basic ',
				(Base64MimeConverter mimeEncode: 
				(self url username unescapePercents, ':', self url password unescapePercents) readStream) upToEnd, 
				String crlf]
! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:39'!
usesSSL

	^ url schemeName = 'https'! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:39'!
waitForConnectionUntil: deadline
	"Wait up until the given deadline for a connection to be established. 
	Return true if it is established by the deadline, false if not."

	[self isConnecting and: [Time millisecondClockValue < deadline]]
		whileTrue: [(Delay forMilliseconds: 100) wait].
	^ self conn isConnected
! !
